@@ -1,39 +1,46 @@
 package Mail::Mbox::MessageParser;
 
 use strict;
+use 5.005;
 use Carp;
 use FileHandle::Unget;
 use File::Spec;
 use File::Temp;
-sub dprint;
+sub _dprint;
 
+use Mail::Mbox::MessageParser::MetaInfo;
 use Mail::Mbox::MessageParser::Config;
 
 use Mail::Mbox::MessageParser::Perl;
 use Mail::Mbox::MessageParser::Grep;
 use Mail::Mbox::MessageParser::Cache;
 
-use vars qw( @ISA $VERSION $DEBUG $UPDATING_CACHE );
+use vars qw( @ISA $VERSION $_DEBUG );
+use vars qw( $_CACHE $UPDATING_CACHE );
 
 @ISA = qw(Exporter);
 
-$VERSION = sprintf "%d.%02d%02d", q/1.21.30/ =~ /(\d+)/g;
-$DEBUG = 0;
+$VERSION = sprintf "%d.%02d%02d", q/1.51.11/ =~ /(\d+)/g;
+$_DEBUG = 0;
 
 #-------------------------------------------------------------------------------
 
-*UPDATING_CACHE = \$Mail::Mbox::MessageParser::Cache::UPDATING_CACHE;
-*SETUP_CACHE = \&Mail::Mbox::MessageParser::Cache::SETUP_CACHE;
-*CLEAR_CACHE = \&Mail::Mbox::MessageParser::Cache::CLEAR_CACHE;
-*WRITE_CACHE = \&Mail::Mbox::MessageParser::Cache::WRITE_CACHE;
+# The class-wide cache, which will be read and written when necessary. i.e.
+# read when an folder reader object is created which uses caching, and
+# written when a different cache is specified, or when the program exits, 
+*_CACHE = \$Mail::Mbox::MessageParser::MetaInfo::_CACHE;
+
+*UPDATING_CACHE = \$Mail::Mbox::MessageParser::MetaInfo::UPDATING_CACHE;
+*SETUP_CACHE = \&Mail::Mbox::MessageParser::MetaInfo::SETUP_CACHE;
+sub SETUP_CACHE;
 
 #-------------------------------------------------------------------------------
 
-# Outputs debug messages if $DEBUG is true. 
+# Outputs debug messages if $_DEBUG is true. 
 
-sub dprint
+sub _dprint
 {
-  return 1 unless $DEBUG;
+  return 1 unless $_DEBUG;
 
   my $message = join '',@_;
 
@@ -42,7 +49,7 @@ sub dprint
     warn "DEBUG (" . __PACKAGE__ . "): $line\n";
   }
 
-  # Be sure to return 1 so code like 'dprint "blah\n" and exit' works.
+  # Be sure to return 1 so code like '_dprint "blah\n" and exit' works.
   return 1;
 }
 
@@ -64,7 +71,7 @@ sub new
     $options->{'enable_grep'} = 0;
   }
 
-  $DEBUG = $options->{'debug'}
+  $_DEBUG = $options->{'debug'}
     if defined $options->{'debug'};
 
   my ($file_type, $need_to_close_filehandle, $error, $endline);
@@ -73,11 +80,14 @@ sub new
     _PREPARE_FILE_HANDLE($options->{'file_name'}, $options->{'file_handle'});
 
   if (defined $error &&
-    !($error eq 'Not a mailbox' && $options->{'force_processing'}))
+    !($error eq 'Not a mailbox' && $options->{'force_processing'}) &&
+    !($error =~ 'Found a mix of unix and Windows line endings' && $options->{'force_processing'})
+    )
   {
-    # Here I assume the only error for which the filehandle was opened is
-    # "Not a mailbox"
-    close $options->{'file_handle'} if $error eq 'Not a mailbox';
+    # Here I assume the only errors for which the filehandle was opened is
+    # "Not a mailbox" and mixed line endings
+    close $options->{'file_handle'}
+      if $error eq 'Not a mailbox' || $error =~ /Found a mix of unix and Windows line endings/;
     return $error;
   }
 
@@ -101,7 +111,7 @@ sub new
 
     if ($UPDATING_CACHE)
     {
-      dprint "Couldn't instantiate Mail::Mbox::MessageParser::Cache: " .
+      _dprint "Couldn't instantiate Mail::Mbox::MessageParser::Cache: " .
         "Updating cache";
       $self = undef;
     }
@@ -115,7 +125,7 @@ sub new
     {
       if ($self =~ /not installed/)
       {
-        dprint "Couldn't instantiate Mail::Mbox::MessageParser::Grep: $self";
+        _dprint "Couldn't instantiate Mail::Mbox::MessageParser::Grep: $self";
       }
       else
       {
@@ -136,7 +146,7 @@ sub new
   die "Couldn't instantiate any mailbox parser implementation"
     unless defined $self;
 
-  dprint "Instantiate mailbox parser implementation: " . ref $self;
+  _dprint "Instantiate mailbox parser implementation: " . ref $self;
 
   $self->_print_debug_information();
 
@@ -151,6 +161,18 @@ sub new
 
 #-------------------------------------------------------------------------------
 
+sub _init
+{
+  my $self = shift;
+
+  $self->{'email_line_number'} = 0;
+  $self->{'email_offset'} = 0;
+  $self->{'email_length'} = 0;
+  $self->{'email_number'} = 0;
+}
+
+#-------------------------------------------------------------------------------
+
 sub DESTROY
 {
   my $self = shift;
@@ -171,7 +193,7 @@ sub _PREPARE_FILE_HANDLE
   my $file_name = shift;
   my $file_handle = shift;
 
-  dprint "Preparing file handle";
+  _dprint "Preparing file handle";
 
   if (defined $file_handle)
   {
@@ -182,7 +204,7 @@ sub _PREPARE_FILE_HANDLE
     binmode $file_handle;
 
     my $file_type = _GET_FILE_TYPE(\$file_handle);
-    dprint "Filehandle file type: $file_type";
+    _dprint "Filehandle file type: $file_type";
 
     # Do decompression if we need to
     if (_IS_COMPRESSED_TYPE($file_type))
@@ -196,29 +218,27 @@ sub _PREPARE_FILE_HANDLE
       return ($decompressed_file_handle,$file_type,0,"Not a mailbox",undef)
         if _GET_FILE_TYPE(\$decompressed_file_handle) ne 'mailbox';
 
-      my $endline = _GET_ENDLINE(\$decompressed_file_handle);
+      my $endline;
+      ($endline, $error) = _GET_ENDLINE(\$decompressed_file_handle);
 
-      return ($decompressed_file_handle,$file_type,0,undef,$endline);
+      return ($decompressed_file_handle,$file_type,0,$error,$endline);
     }
     else
     {
-      dprint "Filehandle is not compressed";
+      _dprint "Filehandle is not compressed";
 
-      return ($file_handle,$file_type,0,"No data on filehandle",undef)
-        if eof($file_handle);
-
-      my $endline = _GET_ENDLINE(\$file_handle);
+      my ($endline, $error) = _GET_ENDLINE(\$file_handle);
 
       return ($file_handle,$file_type,0,"Not a mailbox",$endline)
-        if $file_type ne 'mailbox';
+        if !eof($file_handle) && $file_type ne 'mailbox';
 
-      return ($file_handle,$file_type,0,undef,$endline);
+      return ($file_handle,$file_type,0,$error,$endline);
     }
   }
   else
   {
     my $file_type = _GET_FILE_TYPE(\$file_name);
-    dprint "Filename \"$file_name\" file type: $file_type";
+    _dprint "Filename \"$file_name\" file type: $file_type";
 
     my ($opened_file_handle,$error) =
       _OPEN_FILE_HANDLE($file_name, $file_type);
@@ -226,21 +246,22 @@ sub _PREPARE_FILE_HANDLE
     return ($file_handle,$file_type,0,$error,undef)
       unless defined $opened_file_handle;
 
-    my $endline = _GET_ENDLINE(\$opened_file_handle);
+    my $endline;
+    ($endline, $error) = _GET_ENDLINE(\$opened_file_handle);
 
     if (_IS_COMPRESSED_TYPE($file_type))
     {
       return ($opened_file_handle,$file_type,1,"Not a mailbox",$endline)
         if _GET_FILE_TYPE(\$opened_file_handle) ne 'mailbox';
 
-      return ($opened_file_handle,$file_type,1,undef,$endline);
+      return ($opened_file_handle,$file_type,1,$error,$endline);
     }
     else
     {
       return ($opened_file_handle,$file_type,1,"Not a mailbox",$endline)
         if $file_type ne 'mailbox';
 
-      return ($opened_file_handle,$file_type,1,undef,$endline);
+      return ($opened_file_handle,$file_type,1,$error,$endline);
     }
   }
 }
@@ -254,7 +275,7 @@ sub _OPEN_FILE_HANDLE
   my $file_name = shift;
   my $file_type = shift;
 
-  dprint "Opening file \"$file_name\"";
+  _dprint "Opening file \"$file_name\"";
 
   # Non-compressed file
   unless (_IS_COMPRESSED_TYPE($file_type))
@@ -264,7 +285,7 @@ sub _OPEN_FILE_HANDLE
 
     binmode $file_handle;
 
-    dprint "File \"$file_name\" is not compressed";
+    _dprint "File \"$file_name\" is not compressed";
 
     return ($file_handle,undef);
   }
@@ -273,12 +294,12 @@ sub _OPEN_FILE_HANDLE
   return (undef,"Can't decompress $file_name--no decompressor available")
     unless defined $Mail::Mbox::MessageParser::Config{'programs'}{$file_type};
 
-  my $filter_command = "$Mail::Mbox::MessageParser::Config{'programs'}{$file_type} -cd '$file_name' |";
+  my $filter_command = qq{"$Mail::Mbox::MessageParser::Config{'programs'}{$file_type}" -cd "$file_name" |};
 
-  dprint "Calling \"$filter_command\" to decompress file \"$file_name\".";
+  _dprint "Calling \"$filter_command\" to decompress file \"$file_name\".";
 
-  use vars qw(*OLDSTDERR);
-  open OLDSTDERR,">&STDERR" or die "Can't save STDERR: $!\n";
+  my $oldstderr;
+  open $oldstderr,">&STDERR" or die "Can't save STDERR: $!\n";
   open STDERR,">" . File::Spec->devnull()
     or die "Can't redirect STDERR to " . File::Spec->devnull() . ": $!\n";
 
@@ -289,7 +310,7 @@ sub _OPEN_FILE_HANDLE
 
   binmode $file_handle;
 
-  open STDERR,">&OLDSTDERR" or die "Can't restore STDERR: $!\n";
+  open STDERR, '>&', $oldstderr or die "Can't restore STDERR: $!\n";
 
   if (eof($file_handle))
   {
@@ -302,7 +323,7 @@ sub _OPEN_FILE_HANDLE
 
 #-------------------------------------------------------------------------------
 
-# Returns: unknown, unknown binary, mailbox, non-mailbox ascii, tzip, bzip,
+# Returns: unknown, unknown binary, mailbox, non-mailbox ascii, bzip,
 # bzip2, gzip, compress
 sub _GET_FILE_TYPE
 {
@@ -337,14 +358,14 @@ sub _GET_FILE_TYPE
 
     last unless defined $readResult && $readResult != 0;
 
-    last if _IS_BINARY(\$test_chars);
+    last if _IS_BINARY_MAILBOX(\$test_chars);
 
     if(CORE::length($test_chars) >
         $Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'})
     {
       if(index($test_chars,"\n\n") == -1 && index($test_chars,"\r\n\r\n") == -1)
       {
-        dprint "Couldn't find end of first paragraph after " .
+        _dprint "Couldn't find end of first paragraph after " .
           "$Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'} bytes."
       }
 
@@ -365,29 +386,31 @@ sub _GET_FILE_TYPE
   return 'unknown' unless defined $readResult && $readResult != 0;
 
 
-  unless (_IS_BINARY(\$test_chars))
+  unless (_IS_BINARY_MAILBOX(\$test_chars))
   {
     return 'mailbox' if _IS_MAILBOX(\$test_chars);
     return 'non-mailbox ascii';
   }
 
   # See "magic" on unix systems for details on how to identify file types
-  return 'tzip' if substr($test_chars, 0, 2) eq 'TZ';
   return 'bzip2' if substr($test_chars, 0, 3) eq 'BZh';
   return 'bzip' if substr($test_chars, 0, 2) eq 'BZ';
+  return 'xz' if substr($test_chars, 1, 4) eq '7zXZ';
+  return 'lzip' if substr($test_chars, 0, 4) eq 'LZIP';
 #  return 'zip' if substr($test_chars, 0, 2) eq 'PK' &&
 #    ord(substr($test_chars,3,1)) == 0003 && ord(substr($test_chars,4,1)) == 0004;
   return 'gzip' if
-    ord(substr($test_chars,0,1)) == 0037 && ord(substr($test_chars,1,1)) == 0213;
+    ord(substr($test_chars,0,1)) == oct(37) && ord(substr($test_chars,1,1)) == oct(213);
   return 'compress' if
-    ord(substr($test_chars,0,1)) == 0037 && ord(substr($test_chars,1,1)) == 0235;
+    ord(substr($test_chars,0,1)) == oct(37) && ord(substr($test_chars,1,1)) == oct(235);
 
   return 'unknown binary';
 }
 
 #-------------------------------------------------------------------------------
 
-# Returns: undef, "\r\n", "\n"
+# Returns an endline result of either: undef, "\r\n", "\n"
+# Returns an error message (or undef) as well
 sub _GET_ENDLINE
 {
   my $file_name_or_handle_ref = shift;
@@ -421,14 +444,14 @@ sub _GET_ENDLINE
 
     last unless defined $readResult && $readResult != 0;
 
-    last if _IS_BINARY(\$test_chars);
+    last if _IS_BINARY_MAILBOX(\$test_chars);
 
     if(CORE::length($test_chars) >
         $Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'})
     {
       if(index($test_chars,"\n") == -1 && index($test_chars,"\r\n") == -1)
       {
-        dprint "Couldn't find end of first line after " .
+        _dprint "Couldn't find end of first line after " .
           "$Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'} bytes."
       }
 
@@ -446,17 +469,41 @@ sub _GET_ENDLINE
     $$file_handle_ref->ungets($test_chars);
   }
 
-  return undef unless defined $readResult && $readResult != 0;
+  return undef unless defined $readResult && $readResult != 0; ## no critic (ProhibitExplicitReturnUndef)
 
-  return undef if _IS_BINARY(\$test_chars);
+  return undef if _IS_BINARY_MAILBOX(\$test_chars); ## no critic (ProhibitExplicitReturnUndef)
 
-  if(index($test_chars,"\r\n") != -1)
+  my $windows_count = 0;
+
+  while ($test_chars =~ /\r\n/gs)
   {
-    return "\r\n";
+    $windows_count++;
+  }
+
+  my $unix_count = 0;
+
+  while ($test_chars =~ /(?<!\r)\n/gs)
+  {
+    $unix_count++;
+  }
+
+  _dprint "Found $unix_count UNIX line endings and $windows_count Windows line endings in a sample of length " . 
+    CORE::length($test_chars);
+
+  if($windows_count > 0 && $unix_count == 0)
+  {
+    return "\r\n", undef;
+  }
+  elsif($windows_count == 0 && $unix_count > 0)
+  {
+    return "\n", undef;
   }
   else
   {
-    return "\n";
+    return $windows_count > $unix_count ? "\r\n" : "\n", 'Found a mix of unix and Windows line endings.' .
+      ' Please normalize the line endings using a tool like "dos2unix".' .
+      ' Use the force option to ignore this error and process using ' . ($windows_count > $unix_count ?
+       'Windows' : 'Unix') . ' line endings (best guess).';
   }
 }
 
@@ -468,7 +515,7 @@ sub _IS_COMPRESSED_TYPE
   
   local $" = '|';
 
-  my @types = qw( gzip tzip bzip bzip2 compress );
+  my @types = qw( gzip bzip bzip2 xz lzip compress );
   my $file_type_pattern = "(@types)";
 
   return $file_type =~ /^$file_type_pattern$/;
@@ -478,7 +525,7 @@ sub _IS_COMPRESSED_TYPE
 
 # man perlfork for details
 # simulate open(FOO, "-|")
-sub pipe_from_fork ($)
+sub _pipe_from_fork
 {
   my $parent = shift;
   my $child = new FileHandle::Unget;
@@ -486,7 +533,7 @@ sub pipe_from_fork ($)
   pipe $parent, $child or die;
 
   my $pid = fork();
-  return undef unless defined $pid;
+  return undef unless defined $pid; ## no critic (ProhibitExplicitReturnUndef)
 
   if ($pid)
   {
@@ -511,10 +558,10 @@ sub _DO_WINDOWS_DECOMPRESSION
   return (undef,"Can't decompress file handle--no decompressor available")
     unless defined $Mail::Mbox::MessageParser::Config{'programs'}{$file_type};
 
-  my $filter_command = "$Mail::Mbox::MessageParser::Config{'programs'}{$file_type} -cd";
+  my $filter_command = qq{"$Mail::Mbox::MessageParser::Config{'programs'}{$file_type}" -cd};
 
   my ($temp_file_handle, $temp_file_name) =
-    File::Temp::tempfile('mail-mbox-messageparser-XXXXXX', SUFFIX => '.tmp', UNLINK => 1);
+    File::Temp::tempfile('mail-mbox-messageparser-XXXXXX', SUFFIX => '.tmp', TMPDIR => 1, UNLINK => 1);
 
   while(my $line = <$file_handle>)
   {
@@ -525,7 +572,7 @@ sub _DO_WINDOWS_DECOMPRESSION
   # So that it won't be deleted until the program is complete
   # close $temp_file_handle;
 
-  dprint "Calling \"$filter_command\" to decompress filehandle";
+  _dprint "Calling \"$filter_command\" to decompress filehandle";
 
   my $decompressed_file_handle =
     new FileHandle::Unget("$filter_command $temp_file_name |");
@@ -545,13 +592,13 @@ sub _DO_NONWINDOWS_DECOMPRESSION
   return (undef,"Can't decompress file handle--no decompressor available")
     unless defined $Mail::Mbox::MessageParser::Config{'programs'}{$file_type};
 
-  my $filter_command = "$Mail::Mbox::MessageParser::Config{'programs'}{$file_type} -cd";
+  my $filter_command = qq{"$Mail::Mbox::MessageParser::Config{'programs'}{$file_type}" -cd};
 
-  dprint "Calling \"$filter_command\" to decompress filehandle";
+  _dprint "Calling \"$filter_command\" to decompress filehandle";
 
   # Implicit fork
   my $decompressed_file_handle = new FileHandle::Unget;
-  my $pid = pipe_from_fork($decompressed_file_handle);
+  my $pid = _pipe_from_fork($decompressed_file_handle);
 
   unless (defined($pid))
   {
@@ -566,12 +613,12 @@ sub _DO_NONWINDOWS_DECOMPRESSION
   # uncompressed input.
   unless ($pid)
   {
-    open(FRONT_OF_PIPE, "|$filter_command 2>" . File::Spec->devnull())
+    open(my $front_of_pipe, "|$filter_command 2>" . File::Spec->devnull())
       or return (undef,"Can't execute \"$filter_command\" on file handle: $!");
 
-    binmode FRONT_OF_PIPE;
+    binmode $front_of_pipe;
 
-    print FRONT_OF_PIPE <$file_handle>;
+    print $front_of_pipe (<$file_handle>);
 
     $file_handle->close()
       or return (undef,"Can't execute \"$filter_command\" on file handle: $!");
@@ -579,7 +626,7 @@ sub _DO_NONWINDOWS_DECOMPRESSION
     # We intentionally don't check for error here. This is because the
     # parent may have aborted, in which case we let it take care of
     # error messages. (e.g. Non-mailbox standard input.)
-    close FRONT_OF_PIPE;
+    close $front_of_pipe;
 
     exit;
   }
@@ -609,29 +656,52 @@ sub _DO_DECOMPRESSION
 
 #-------------------------------------------------------------------------------
 
-# Simulates -B, which consumes data on a stream.
-sub _IS_BINARY
+# Simulates -B, which consumes data on a stream. We only look at the first
+# 1000 characters because the body may have foreign binary-like characters
+sub _IS_BINARY_MAILBOX
 {
-  my $data_length = CORE::length(${$_[0]});
-  my $bin_length = ${$_[0]} =~ tr/[\t\n\x20-\x7e]//c;
+  my ($start, $data_length);
+
+  # Unix line endings
+  {
+    $start = 0;
+
+    # Handle newlines at the start
+    while (index(${$_[0]}, "\n\n", $start) == $start) {
+      $start += 2;
+    }
+
+    $data_length = index(${$_[0]}, "\n\n", $start) - $start;
+  }
+
+  # If we didn't succeed with Unix line endings, try DOS line endings
+  if ($data_length == -1)
+  {
+    # Handle newlines at the start
+    $start = 0;
+
+    while (index(${$_[0]}, "\r\n\r\n", $start) == $start) {
+      $start += 4;
+    }
+
+    $data_length = index(${$_[0]}, "\r\n\r\n", $start) - $start;
+  }
+
+  # Didn't find any kind of empty line. Use the whole buffer.
+  $data_length = CORE::length(${$_[0]}) - $start if $data_length == -1;
+
+  my $bin_length = substr(${$_[0]}, $start ,$data_length) =~ tr/[\t\n\x20-\x7e]//c;
+
   my $non_bin_length = $data_length - $bin_length;
 
-  if (($non_bin_length / $data_length) > .70)
-  {
-    return 0;
-  }
-  else
-  {
-    return 1;
-  }
+  return (($non_bin_length / $data_length) <= .70);
 }
 
 #-------------------------------------------------------------------------------
 
-# Detects whether an ASCII file is a mailbox, based on whether it has
-# a line whose prefix is 'From' or 'X-From-Line:' or 'X-Draft-From:',
-# and another line whose prefix is 'Received ', 'Date:', 'Subject:',
-# 'X-Status:', 'Status:', or 'To:'.
+# Detects whether an ASCII file is a mailbox, based on whether it has a line
+# whose prefix is 'From' and another line whose prefix is 'Received ',
+# 'Date:', 'Subject:', 'X-Status:', 'Status:', or 'To:'.
 
 sub _IS_MAILBOX
 {
@@ -652,7 +722,32 @@ sub _IS_MAILBOX
 
 sub reset
 {
-  die "Derived class must provide an implementation";
+  my $self = shift;
+
+  if (_IS_A_PIPE($self->{'file_handle'}))
+  {
+    _dprint "Avoiding seek() on a pipe";
+  }
+  else
+  {
+    seek $self->{'file_handle'}, length($self->{'prologue'}), 0
+  }
+
+  $self->{'email_line_number'} = 0;
+  $self->{'email_offset'} = 0;
+  $self->{'email_length'} = 0;
+  $self->{'email_number'} = 0;
+}
+
+#-------------------------------------------------------------------------------
+
+# Ceci n'set pas une pipe
+sub _IS_A_PIPE
+{
+  my $file_handle = shift;
+
+  return (-t $file_handle || -S $file_handle || -p $file_handle || ## no critic (ProhibitInteractiveTest)
+    !-f $file_handle || !(seek $file_handle, 0, 1));
 }
 
 #-------------------------------------------------------------------------------
@@ -677,11 +772,11 @@ sub prologue
 
 sub _print_debug_information
 {
-  return unless $DEBUG;
+  return unless $_DEBUG;
 
   my $self = shift;
 
-  dprint "Version: $VERSION";
+  _dprint "Version: $VERSION";
 
   foreach my $key (keys %$self)
   {
@@ -695,7 +790,7 @@ sub _print_debug_information
       $value = '<undef>';
     }
 
-    dprint "$key: $value";
+    _dprint "$key: $value";
   }
 }
 
@@ -706,7 +801,11 @@ sub end_of_file
 {
   my $self = shift;
 
-  return $self->{'end_of_file'};
+  # Reset eof in case the file was appended to. Hopefully this works all the
+  # time. See perldoc -f seek for details.
+  seek($self->{'file_handle'},0,1) if eof $self->{'file_handle'};
+
+  return eof $self->{'file_handle'};
 }
 
 #-------------------------------------------------------------------------------
@@ -763,22 +862,77 @@ sub read_next_email
 
   if ($UPDATING_CACHE)
   {
-    dprint "Storing data into cache, length " . $self->{'email_length'};
+    _dprint "Storing data into cache, length " . $self->{'email_length'};
 
-    my $cache = $Mail::Mbox::MessageParser::Cache::CACHE;
+    my $_CACHE = $Mail::Mbox::MessageParser::Cache::_CACHE;
 
-    $cache->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}-1] =
+    $_CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'length'} =
       $self->{'email_length'};
 
-    $cache->{$self->{'file_name'}}{'line_numbers'}[$self->{'email_number'}-1] =
+    $_CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'line_number'} =
       $self->{'email_line_number'};
 
-    $cache->{$self->{'file_name'}}{'offsets'}[$self->{'email_number'}-1] =
+    $_CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'offset'} =
       $self->{'email_offset'};
+    $_CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'validated'} =
+      1;
 
-    $Mail::Mbox::MessageParser::Cache::CACHE_MODIFIED = 1;
+    $_CACHE->{$self->{'file_name'}}{'modified'} = 1;
+
+    if ($self->end_of_file())
+    {
+      $UPDATING_CACHE = 0;
+
+      # Last one is always validated
+      $_CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'validated'} =
+        1;
+    }
+
+  }
+}
+
+#-------------------------------------------------------------------------------
+
+# - Returns header lines in the email header which match the given name.
+# - Example names: 'From:', 'Received:' or 'From '
+# - If the calling context wants a list, a list of the matching header lines
+#   are returned. Otherwise, the first (and perhaps only) match is returned.
+# - Wrapped lines are handled. Look for multiple \n's in the return value(s)
+# - 'From ' also looks for Gnus 'X-From-Line:' or 'X-Draft-From:'
+
+# Stolen from grepmail
+sub _GET_HEADER_FIELD
+{
+  my $email_header = shift;
+  my $header_name = shift;
+  my $endline = shift;
+
+  die unless ref $email_header;
+
+  # Avoid perl 5.6 bug which causes spurious warning even though $email_header
+  # is defined.
+  local $^W = 0 if $] >= 5.006 && $] < 5.008;
+
+  if ($header_name =~ /^From$/i &&
+    $$email_header =~ /^((?:From\s|X-From-Line:|X-Draft-From:).*$endline(\s.*$endline)*)/im)
+  {
+    return wantarray ? ($1) : $1;
   }
 
+  my @matches = $$email_header =~ /^($header_name\s.*$endline(?:\s.*$endline)*)/igm;
+
+  if (@matches)
+  {
+    return wantarray ? @matches : shift @matches;
+  }
+
+  if (lc $header_name eq 'from ' &&
+    $$email_header =~ /^(From\s.*$endline(\s.*$endline)*)/im)
+  {
+    return wantarray ? ($1) : $1;
+  }
+
+  return undef; ## no critic (ProhibitExplicitReturnUndef)
 }
 
 1;
@@ -797,7 +951,8 @@ Mail::Mbox::MessageParser - A fast and simple mbox folder reader
 
   use Mail::Mbox::MessageParser;
 
-  my $file_name = 'mail/saved-mail';
+  # Compression support
+  my $file_name = 'mail/saved-mail.xz';
   my $file_handle = new FileHandle($file_name);
 
   # Set up cache. (Not necessary if enable_cache is false.)
@@ -812,6 +967,8 @@ Mail::Mbox::MessageParser - A fast and simple mbox folder reader
       'enable_grep' => 1,
     } );
 
+  die $folder_reader unless ref $folder_reader;
+
   # Any newlines or such before the start of the first email
   my $prologue = $folder_reader->prologue;
   print $prologue;
@@ -883,11 +1040,10 @@ file using the file name. You should always pass the file name if you have it,
 so that the parser can cache the mailbox information.
 
 This module will automatically decompress the mailbox as necessary. If a
-filename is available but the file handle is undef, the module will call
-either tzip, bzip2, or gzip to decompress the file in memory if the filename
-ends with .tz, .bz2, or .gz, respectively. If the file handle is defined, it
-will detect the type of compression and apply the correct decompression
-program.
+filename is available but the file handle is undef, the module will call bzip,
+bzip2, gzip, lzip, xz to decompress the file in memory if the filename ends
+with the appropriate suffix. If the file handle is defined, it will detect the
+type of compression and apply the correct decompression program.
 
 The Cache, Grep, or Perl implementation of the parser will be loaded,
 whichever is most appropriate. For example, the first time you use caching,
@@ -952,7 +1108,7 @@ Returns the byte offset of the last email read.
 =item read_next_email()
 
 Returns a reference to a scalar holding the text of the next email in the
-mailbox.
+mailbox, or undef at the end of the file.
 
 =back
 
@@ -971,8 +1127,8 @@ David Coppit <david@coppit.org>.
 
 =head1 LICENSE
 
-This software is distributed under the terms of the GPL. See the file
-"LICENSE" for more information.
+This code is distributed under the GNU General Public License (GPL) Version 2.
+See the file LICENSE in the distribution for details.
 
 
 =head1 HISTORY
